import os
import openmc
import numpy as np
import typing
from tempfile import mkdtemp
from pathlib import Path
import matplotlib.pyplot as plt

from PIL import Image

from numpy import asarray


def get_rgb_from_int(value: int) -> typing.Tuple[int, int, int]:
    blue = value & 255
    green = (value >> 8) & 255
    red = (value >> 16) & 255
    return red, green, blue


def get_int_from_rgb(rgb: typing.Tuple[int, int, int]) -> int:
    red = rgb[0]
    green = rgb[1]
    blue = rgb[2]
    return (red << 16) + (green << 8) + blue


def get_plot_extent(
    self, plot_left, plot_right, plot_bottom, plot_top, slice_value, bb, view_direction
):
    if view_direction not in ["x", "y", "z"]:
        raise ValueError('view_direction must be "x", "y" or "z"')

    if plot_left is None:
        plot_left = self.get_side_extent(
            side="left", bounding_box=bb, view_direction=view_direction
        )

    if plot_right is None:
        plot_right = self.get_side_extent(
            side="right", bounding_box=bb, view_direction=view_direction
        )

    if plot_bottom is None:
        plot_bottom = self.get_side_extent(
            side="bottom", bounding_box=bb, view_direction=view_direction
        )

    if plot_top is None:
        plot_top = self.get_side_extent(
            side="top", bounding_box=bb, view_direction=view_direction
        )

    if slice_value is None:
        slice_value = self.get_mid_slice_value(
            bounding_box=bb, view_direction=view_direction
        )
    return plot_left, plot_right, plot_bottom, plot_top, slice_value


def get_side_extent(self, side: str, view_direction: str, bounding_box=None):
    if bounding_box is None:
        bounding_box = self.bounding_box

    avail_extents = {}
    avail_extents[("left", "x")] = bounding_box[0][1]
    avail_extents[("right", "x")] = bounding_box[1][1]
    avail_extents[("top", "x")] = bounding_box[1][2]
    avail_extents[("bottom", "x")] = bounding_box[0][2]
    avail_extents[("left", "y")] = bounding_box[0][0]
    avail_extents[("right", "y")] = bounding_box[1][0]
    avail_extents[("top", "y")] = bounding_box[1][2]
    avail_extents[("bottom", "y")] = bounding_box[0][2]
    avail_extents[("left", "z")] = bounding_box[0][0]
    avail_extents[("right", "z")] = bounding_box[1][0]
    avail_extents[("top", "z")] = bounding_box[1][1]
    avail_extents[("bottom", "z")] = bounding_box[0][1]
    return avail_extents[(side, view_direction)]


def get_mpl_plot_extent(self, view_direction):
    """Returns the (x_min, x_max, y_min, y_max) of the bounding box. The
    view_direction is taken into account and can be set using
    openmc.Geometry.view_direction property is taken into account and can be
    set to 'x', 'y' or 'z'."""

    bb = self.bounding_box

    x_min = self.get_side_extent(
        side="left", view_direction=view_direction, bounding_box=bb
    )
    x_max = self.get_side_extent(
        side="right", view_direction=view_direction, bounding_box=bb
    )
    y_min = self.get_side_extent(
        side="bottom", view_direction=view_direction, bounding_box=bb
    )
    y_max = self.get_side_extent(
        side="top", view_direction=view_direction, bounding_box=bb
    )

    return (x_min, x_max, y_min, y_max)


def get_mid_slice_value(self, view_direction, bounding_box=None):
    """Returns the position of the center of the mesh. The view_direction is
    taken into account and can be set using openmc.Geometry.view_direction
    property is taken into account and can be set to 'x', 'y' or 'z'."""

    if bounding_box is None:
        bounding_box = self.bounding_box

    if view_direction == "x":
        plot_edge = (bounding_box[0][0] + bounding_box[1][0]) / 2
    elif view_direction == "y":
        plot_edge = (bounding_box[0][1] + bounding_box[1][1]) / 2
    elif view_direction == "z":
        plot_edge = (bounding_box[0][2] + bounding_box[1][2]) / 2
    else:
        msg = f'view_direction must be "x", "y" or "z" not {view_direction}'
        raise ValueError(msg)

    if np.isinf(plot_edge):
        msg = f"Mid slice value can't be obtained from the bounding box as boundary is at inf. Slice value must be specified by user"
        raise ValueError(msg)

    return plot_edge


def get_axis_labels(self, view_direction):
    """Returns two axis label values for the x and y value. Takes
    view_direction into account."""

    if view_direction == "x":
        xlabel = "Y [cm]"
        ylabel = "Z [cm]"
    if view_direction == "y":
        xlabel = "X [cm]"
        ylabel = "Z [cm]"
    if view_direction == "z":
        xlabel = "X [cm]"
        ylabel = "Y [cm]"
    return xlabel, ylabel


def find_cell_id(self, inputs):
    plot_x, plot_y, slice_value = inputs
    return self.find((plot_x, plot_y, slice_value))


def is_geometry_dagmc(self):
    for univ in self.get_all_universes().values():
        if isinstance(univ, openmc.DAGMCUniverse):
            return True
    return False


def get_dagmc_filepath(self):
    "absolute path"
    for univ in self.get_all_universes().values():
        if isinstance(univ, openmc.DAGMCUniverse):
            return Path(univ.filename).absolute()
            # could try relative paths for more complex filenames
            # return Path(univ.filename).relative_to(Path(__file__).parent)


def get_dagmc_universe(self):
    for univ in self.get_all_universes().values():
        if isinstance(univ, openmc.DAGMCUniverse):
            return univ


def get_slice_of_material_ids(
    self,
    view_direction: str,
    slice_value: typing.Optional[float] = None,
    plot_top: typing.Optional[float] = None,
    plot_bottom: typing.Optional[float] = None,
    plot_left: typing.Optional[float] = None,
    plot_right: typing.Optional[float] = None,
    pixels_across: int = 500,
    verbose: bool = False,
):
    """Returns a grid of material IDs for each mesh voxel on the slice. This
    can be passed directly to plotting functions like Matplotlib imshow. 0
    values represent void space or undefined space.

    Args:
        slice_value:
        plot_top:
        plot_bottom:
        plot_left:
        plot_right:
        pixels_across:
        verbose:
    """

    tmp_folder = mkdtemp(prefix="openmc_geometry_plotter_tmp_files_")
    if verbose:
        print(f"writing files to {tmp_folder}")

    if self.is_geometry_dagmc():
        dagmc_abs_filepath = self.get_dagmc_filepath()

        os.system(f"cp {dagmc_abs_filepath} {tmp_folder}")

        dag_universe = self.get_dagmc_universe()

        if str(Path(dag_universe.filename).name) != str(Path(dag_universe.filename)):
            msg = (
                "Paths for dagmc files that contain folders are not currently "
                "supported. Try setting your DAGMCUniverse.filename "
                f"to {Path(dag_universe.filename).name} instead of "
                f"{Path(dag_universe.filename)}"
            )
            raise IsADirectoryError(msg)
        # dag_universe.filename = dagmc_abs_filepath.name

        # dagmuniverse does not have a get_all_materials
        mat_names = dag_universe.material_names

        # mat ids are not known by the dagmc file
        # assumed mat ids start at 1 and continue,
        # universe.n_cells is the equivilent approximation for cell ids
        mat_ids = range(1, len(mat_names) + 1)

        # if any of the plot_ are None then this needs calculating
        # might need to be self.bounding_box
        bb = dag_universe.bounding_box

    else:
        original_materials = self.get_all_materials()
        mat_ids = original_materials.keys()

        mat_names = []
        for key, value in original_materials.items():
            mat_names.append(value.name)

        # if any of the plot_ are None then this needs calculating
        bb = self.bounding_box

    self.export_to_xml(tmp_folder)

    plot_left, plot_right, plot_bottom, plot_top, slice_value = self.get_plot_extent(
        plot_left, plot_right, plot_bottom, plot_top, slice_value, bb, view_direction
    )

    plot_width = abs(plot_left - plot_right)
    plot_height = abs(plot_bottom - plot_top)

    aspect_ratio = plot_height / plot_width
    pixels_up = int(pixels_across * aspect_ratio)

    all_materials = []
    for i, n in zip(mat_ids, mat_names):
        new_mat = openmc.Material()
        new_mat.id = i
        new_mat.name = n
        new_mat.add_nuclide("He4", 1)
        all_materials.append(new_mat)
    nn = openmc.Materials(all_materials)
    nn.export_to_xml(tmp_folder)

    my_settings = openmc.Settings()
    my_settings.output = {"summary": False, "tallies": False}
    # add verbose setting to avoid print out
    my_settings.particles = 1
    my_settings.batches = 1
    my_settings.run_mode = "fixed source"
    my_settings.export_to_xml(tmp_folder)

    my_plot = openmc.Plot()

    plot_x = (plot_left + plot_right) / 2
    plot_y = (plot_top + plot_bottom) / 2

    width = plot_left - plot_right
    height = plot_top - plot_bottom
    my_plot.width = (width, height)

    if view_direction == "z":
        my_plot.basis = "xy"
        my_plot.origin = (plot_x, plot_y, slice_value)
    if view_direction == "x":
        my_plot.basis = "yz"
        my_plot.origin = (slice_value, plot_x, plot_y)
    if view_direction == "y":
        my_plot.basis = "xz"
        my_plot.origin = (plot_x, slice_value, plot_y)

    my_plot.pixels = (pixels_across, pixels_up)
    colors_dict = {}
    for mat_id in mat_ids:
        colors_dict[mat_id] = get_rgb_from_int(mat_id)
    my_plot.colors = colors_dict
    my_plot.background = (0, 0, 0)  # void material is 0
    my_plot.color_by = "material"
    my_plot.id = 42  # the integer used to name of the plot_1.ppm file
    my_plots = openmc.Plots([my_plot])
    my_plots.export_to_xml(tmp_folder)

    # TODO unset this afterwards
    package_dir = Path(__file__).parent
    openmc.config["cross_sections"] = package_dir / "cross_sections.xml"

    openmc.plot_geometry(cwd=tmp_folder, output=verbose)

    if verbose:
        print(f"Temporary image and xml files written to {tmp_folder}")

    # load the image
    if (Path(tmp_folder) / f"plot_{my_plot.id}.ppm").is_file():
        image = Image.open(Path(tmp_folder) / f"plot_{my_plot.id}.ppm")
    elif (Path(tmp_folder) / f"plot_{my_plot.id}.png").is_file():
        image = Image.open(Path(tmp_folder) / f"plot_{my_plot.id}.png")
    else:
        raise FileNotFoundError(f"openmc plot mode image was not found in {tmp_folder}")

    # convert the image to a numpy array
    image_values = asarray(image)

    # the image_values have three entries for RGB but we just need one.
    # this reduces the nested list to contain a single value per pixel
    # image_value = [
    #     [inner_entry[0] for inner_entry in outer_entry] for outer_entry in image_values
    # ]
    image_value = [
        [get_int_from_rgb(inner_entry) for inner_entry in outer_entry]
        for outer_entry in image_values
    ]

    # replaces rgb (255,255,255) which is 16777215 values with 0.
    # 0 is the color for void space
    # (255,255,255) gets returned by undefined regions outside the geometry
    trimmed_image_value = [
        [0 if x == 16777215 else x for x in inner_list] for inner_list in image_value
    ]

    return trimmed_image_value


def get_slice_of_cell_ids(
    self,
    view_direction: str,
    slice_value: typing.Optional[float] = None,
    plot_top: typing.Optional[float] = None,
    plot_bottom: typing.Optional[float] = None,
    plot_left: typing.Optional[float] = None,
    plot_right: typing.Optional[float] = None,
    pixels_across: int = 500,
    verbose: bool = False,
):
    """Returns a grid of cell IDs for each mesh voxel on the slice. This
    can be passed directly to plotting functions like Matplotlib imshow. 0
    values represent void space or undefined space.

    Args:
        slice_value:
        plot_top:
        plot_bottom:
        plot_left:
        plot_right:
        pixels_across:
        verbose:
    """

    tmp_folder = mkdtemp(prefix="openmc_geometry_plotter_tmp_files_")
    if verbose:
        print(f"writing files to {tmp_folder}")

    if self.is_geometry_dagmc():
        dagmc_abs_filepath = self.get_dagmc_filepath()

        os.system(f"cp {dagmc_abs_filepath} {tmp_folder}")

        dag_universe = self.get_dagmc_universe()

        if str(Path(dag_universe.filename).name) != str(Path(dag_universe.filename)):
            msg = (
                "Paths for dagmc files that contain folders are not currently "
                "supported. Try setting your DAGMCUniverse.filename "
                f"to {Path(dag_universe.filename).name} instead of "
                f"{Path(dag_universe.filename)}"
            )
            raise IsADirectoryError(msg)
        # dag_universe.filename = dagmc_abs_filepath.name

        # dagmuniverse does not have a get_all_materials
        mat_names = dag_universe.material_names

        # mat ids are not known by the dagmc file
        # assumed mat ids start at 1 and continue,
        # universe.n_cells is the equivilent approximation for cell ids
        mat_ids = range(1, len(mat_names) + 1)

        # if any of the plot_ are None then this needs calculating
        # might need to be self.bounding_box
        bb = dag_universe.bounding_box

    else:

        original_materials = self.get_all_materials()
        mat_ids = original_materials.keys()

        mat_names = []
        for key, value in original_materials.items():
            mat_names.append(value.name)

        bb = self.bounding_box

    self.export_to_xml(tmp_folder)

    plot_left, plot_right, plot_bottom, plot_top, slice_value = self.get_plot_extent(
        plot_left, plot_right, plot_bottom, plot_top, slice_value, bb, view_direction
    )

    plot_width = abs(plot_left - plot_right)
    plot_height = abs(plot_bottom - plot_top)

    aspect_ratio = plot_height / plot_width
    pixels_up = int(pixels_across * aspect_ratio)

    # might not work for dagmc model
    cell_ids = self.get_all_cells().keys()

    all_materials = []
    for i, n in zip(mat_ids, mat_names):
        new_mat = openmc.Material()
        new_mat.id = i
        new_mat.name = n
        new_mat.add_nuclide("He4", 1)
        all_materials.append(new_mat)
    nn = openmc.Materials(all_materials)
    nn.export_to_xml(tmp_folder)

    my_settings = openmc.Settings()
    my_settings.output = {"summary": False, "tallies": False}
    my_settings.particles = 1
    my_settings.batches = 1
    my_settings.run_mode = "fixed source"
    my_settings.export_to_xml(tmp_folder)

    my_plot = openmc.Plot()

    plot_x = (plot_left + plot_right) / 2
    plot_y = (plot_top + plot_bottom) / 2

    width = plot_left - plot_right
    height = plot_top - plot_bottom
    my_plot.width = (width, height)

    if view_direction == "z":
        my_plot.basis = "xy"
        my_plot.origin = (plot_x, plot_y, slice_value)
    if view_direction == "x":
        my_plot.basis = "yz"
        my_plot.origin = (slice_value, plot_x, plot_y)
    if view_direction == "y":
        my_plot.basis = "xz"
        my_plot.origin = (plot_x, slice_value, plot_y)

    my_plot.pixels = (pixels_across, pixels_up)
    # my_plot.pixels = (100,100)
    colors_dict = {}
    for cell_id in cell_ids:
        # TODO make use of rgb to int convertor
        colors_dict[cell_id] = get_rgb_from_int(cell_id)
    my_plot.colors = colors_dict
    my_plot.background = (0, 0, 0)  # void material is 0
    my_plot.color_by = "cell"
    my_plot.id = 24  # the integer used to name of the plot_1.ppm file
    my_plots = openmc.Plots([my_plot])
    my_plots.export_to_xml(tmp_folder)

    # TODO unset this afterwards
    package_dir = Path(__file__).parent
    openmc.config["cross_sections"] = package_dir / "cross_sections.xml"

    openmc.plot_geometry(cwd=tmp_folder, output=verbose)
    if verbose:
        print(f"Temporary image and xml files written to {tmp_folder}")

    # load the image
    if (Path(tmp_folder) / f"plot_{my_plot.id}.ppm").is_file():
        image = Image.open(Path(tmp_folder) / f"plot_{my_plot.id}.ppm")
    elif (Path(tmp_folder) / f"plot_{my_plot.id}.png").is_file():
        image = Image.open(Path(tmp_folder) / f"plot_{my_plot.id}.png")
    else:
        raise FileNotFoundError(f"openmc plot mode image was not found in {tmp_folder}")

    # convert the image to a numpy array
    image_values = asarray(image)

    # the image_values have three entries for RGB but we just need one.
    # this reduces the nested list to contain a single value per pixel
    image_value = [
        [get_int_from_rgb(inner_entry) for inner_entry in outer_entry]
        for outer_entry in image_values
    ]

    # replaces rgb (255,255,255) which is 16777215 values with 0.
    # 0 is the color for void space
    # (255,255,255) gets returned by undefined regions outside the geometry
    trimmed_image_value = [
        [0 if x == 16777215 else x for x in inner_list] for inner_list in image_value
    ]

    return trimmed_image_value


# patching openmc

openmc.Geometry.get_dagmc_universe = get_dagmc_universe
openmc.Geometry.is_geometry_dagmc = is_geometry_dagmc
openmc.Geometry.get_dagmc_filepath = get_dagmc_filepath
openmc.Geometry.get_plot_extent = get_plot_extent
openmc.Geometry.get_side_extent = get_side_extent
openmc.Geometry.get_mpl_plot_extent = get_mpl_plot_extent
openmc.Geometry.get_mid_slice_value = get_mid_slice_value
openmc.Geometry.get_axis_labels = get_axis_labels
openmc.Geometry.get_slice_of_material_ids = get_slice_of_material_ids
openmc.Geometry.get_slice_of_cell_ids = get_slice_of_cell_ids
openmc.Geometry.find_cell_id = find_cell_id


openmc.DAGMCUniverse.get_all_universes = openmc.Universe.get_all_universes



import runpy
import sys
import openmc_geometry_plot
from pathlib import Path


def main():
    path_to_app = str(Path(openmc_geometry_plot.__path__[0]) / "app.py")

    sys.argv = ["streamlit", "run", path_to_app]
    runpy.run_module("streamlit", run_name="__main__")


if __name__ == "__main__":
    main()



import matplotlib.patches as mpatches
import matplotlib.pylab as plt
import numpy as np
import openmc
import openmc_geometry_plot  # adds functions to openmc.Geometry
from matplotlib import colors

# MATERIALS

mat = openmc.Material()
mat.set_density("g/cm3", 1)
mat.id = 1
mat.name = "lead"
mat.add_element("Pb", 1)

mats = openmc.Materials([mat])

# GEOMETRY

# surfaces
surf1 = openmc.Sphere(r=100)
surf2 = openmc.Sphere(r=200)
surf3 = openmc.Sphere(r=300, boundary_type="vacuum")

# regions
region1 = -surf1
region2 = +surf1 & -surf2
region3 = +surf2 & -surf3

# cells
cell1 = openmc.Cell(region=region1)
cell1.id = 1
cell1.name = "cell 1"
# cell 1 is not filled with any material
cell2 = openmc.Cell(region=region2)
cell2.id = 2
cell2.fill = mat
cell2.name = "cell 2"
cell3 = openmc.Cell(region=region3)
cell3.fill = mat
cell3.id = 3
cell3.name = "cell 3"

my_geometry = openmc.Geometry([cell1, cell2, cell3])

fig, (ax1, ax2) = plt.subplots(1, 2)

# these three lines are functionality added by the openmc_geometry_plot
xlabel, ylabel = my_geometry.get_axis_labels(view_direction="x")

ax1.set_xlabel(xlabel)
ax1.set_ylabel(ylabel)
ax1.set_title("material ids")
ax2.set_xlabel(xlabel)
ax2.set_ylabel(ylabel)
ax2.set_title("cell ids")

mat_id_slice = my_geometry.get_slice_of_material_ids(
    view_direction="x", pixels_across=600
)
cell_id_slice = my_geometry.get_slice_of_cell_ids(view_direction="x", pixels_across=600)

# gets unique levels for outlines contour plot and for the color scale
mat_levels = np.unique([item for sublist in mat_id_slice for item in sublist])
cell_levels = np.unique([item for sublist in cell_id_slice for item in sublist])

mat_cmap = colors.ListedColormap(["white", "red"])
# our material ids are set to be 1 and 2. void space is 0
mat_bounds = [0, 1, 2]
mat_norm = colors.BoundaryNorm(mat_bounds, mat_cmap.N)

cell_cmap = colors.ListedColormap(["white", "blue", "green", "yellow"])
# our cell ids are set to be 1, 2 and 3. void space or undefined is 0
cell_bounds = [0, 1, 2, 3, 4]
cell_norm = colors.BoundaryNorm(cell_bounds, cell_cmap.N)

# gets the extent of the geometry to pass to matplotlib
plot_extent = my_geometry.get_mpl_plot_extent(view_direction="x")

# shows the mat ids with selected colors
im_mat = ax1.imshow(
    mat_id_slice,
    extent=plot_extent,
    interpolation="none",
    norm=mat_norm,  # needed for colors
    cmap=mat_cmap,  # needed for colors
)

# shows the cell ids with selected colors
im_cell = ax2.imshow(
    cell_id_slice,
    extent=plot_extent,
    interpolation="none",
    norm=cell_norm,  # needed for colors
    cmap=cell_cmap,  # needed for colors
)

# adds contour lines on material boundaries
ax1.contour(
    mat_id_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=mat_levels,
    linewidths=1,
    extent=plot_extent,
)
# adds contour lines on cell boundaries
ax2.contour(
    cell_id_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=cell_levels,
    linewidths=1,
    extent=plot_extent,
)

# code from here onwards adds a legend for both materials and cells

# gets the colors of the values according to the color map used by imshow
colors_mat = [im_mat.cmap(im_mat.norm(value)) for value in mat_levels]
colors_cells = [im_cell.cmap(im_cell.norm(value)) for value in cell_levels]

# these will be the labels used for each legend entry
mat_labels = {1: "mat1 lead"}
cell_labels = {1: "cell 1", 2: "cell 2", 3: "cell 3"}

mat_patches = [
    mpatches.Patch(color=colors_mat[i], label=mat_labels[mat_levels[i]])
    for i in range(len(mat_levels))[1:]  # skips the first 0 value
]
cell_patches = [
    mpatches.Patch(color=colors_cells[i], label=cell_labels[cell_levels[i]])
    for i in range(len(cell_levels))[1:]  # skips the first 0 value
]

ax1.legend(handles=mat_patches, bbox_to_anchor=(0.5, -0.55), loc="lower center")
ax2.legend(handles=cell_patches, bbox_to_anchor=(0.5, -0.55), loc="lower center")

plt.show()



import openmc
import openmc_geometry_plot
from pathlib import Path
import numpy as np
import pytest


def test_is_geometry_dagmc():
    bound_dag_univ = openmc.DAGMCUniverse(
        filename="two_disconnected_cubes.h5m"
    ).bounded_universe()
    my_geometry = openmc.Geometry(root=bound_dag_univ)
    assert my_geometry.is_geometry_dagmc() is True

    # same as above but no bounding cell
    bound_dag_univ = openmc.DAGMCUniverse(filename="two_disconnected_cubes.h5m")
    my_geometry = openmc.Geometry(root=bound_dag_univ)
    assert my_geometry.is_geometry_dagmc() is True


def test_get_dagmc_filepath():
    bound_dag_univ = openmc.DAGMCUniverse(filename="two_disconnected_cubes.h5m")
    my_geometry = openmc.Geometry(root=bound_dag_univ)
    assert (
        my_geometry.get_dagmc_filepath()
        == Path(__file__).parent / "two_disconnected_cubes.h5m"
    )


def test_get_dagmc_universe():
    bound_dag_univ = openmc.DAGMCUniverse(filename="two_disconnected_cubes.h5m")
    my_geometry = openmc.Geometry(root=bound_dag_univ)
    assert isinstance(my_geometry.get_dagmc_universe(), openmc.DAGMCUniverse)


# TODO add check that the numbers returned are correct
def test_slice_material_dagmc_file():
    bound_dag_univ = openmc.DAGMCUniverse(
        filename="two_disconnected_cubes.h5m"
    ).bounded_universe()
    my_geometry = openmc.Geometry(root=bound_dag_univ)

    data_slice = my_geometry.get_slice_of_material_ids(
        view_direction="x", slice_value=1, pixels_across=10
    )

    assert np.array(data_slice).shape == (10, 10)


# TODO add check that the numbers returned are correct
def test_slice_cell_dagmc_file():
    bound_dag_univ = openmc.DAGMCUniverse(
        filename="two_disconnected_cubes.h5m"
    ).bounded_universe()
    my_geometry = openmc.Geometry(root=bound_dag_univ)

    data_slice = my_geometry.get_slice_of_cell_ids(
        view_direction="x", slice_value=1, pixels_across=10
    )

    assert np.array(data_slice).shape == (10, 10)


# future test to check that use of dagmc directories raise an error. as they are not supported
# with pytest.raises(ValueError):
#     data_slice = my_geometry.get_slice_of_cell_ids(
#         view_direction="x", slice_value=1, pixels_across=10
#     )



from pathlib import Path

import matplotlib.pylab as plt
import numpy as np
import openmc
import openmc_geometry_plot  # adds plot_axis_slice to openmc.Geometry

# minimal openmc.Geoemtry made just from a single DAGMC file with a bounding surface
bound_dag_univ = openmc.DAGMCUniverse(
    filename="dagmc_180_tokamak.h5m"
).bounded_universe()
my_geometry = openmc.Geometry(root=bound_dag_univ)


# example code for plotting materials of the geometry with an outline

data_slice = my_geometry.get_slice_of_material_ids(
    view_direction="y", slice_value=1, pixels_across=1000
)

xlabel, ylabel = my_geometry.get_axis_labels(view_direction="y")
plt.xlabel(xlabel)
plt.ylabel(ylabel)

plot_extent = my_geometry.get_mpl_plot_extent(view_direction="y")

# shows the materials in random colors
plt.imshow(
    data_slice,
    extent=plot_extent,
    interpolation="none",
)

# gets unique levels for outlines contour plot
levels = np.unique([item for sublist in data_slice for item in sublist])

# shows the material outlines
plt.contour(
    data_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=levels,
    linewidths=2,
    extent=plot_extent,
)

plt.show()



import xml.etree.ElementTree as ET
from pathlib import Path
import plotly.graph_objects as go
import matplotlib.pyplot as plt
import openmc
import streamlit as st
from matplotlib import colors
from pylab import cm, colormaps  # *
import numpy as np

# import dagmc_h5m_file_inspector as di

import openmc_geometry_plot  # adds extra functions to openmc.Geometry


def save_uploadedfile(uploadedfile):
    with open(uploadedfile.name, "wb") as f:
        f.write(uploadedfile.getbuffer())
    return st.success(f"Saved File to {uploadedfile.name}")


def header():
    """This section writes out the page header common to all tabs"""

    st.set_page_config(
        page_title="OpenMC Geometry Plot",
        page_icon="⚛",
        layout="wide",
    )

    hide_streamlit_style = """
                <style>
                #MainMenu {visibility: hidden;}
                footer {
                    visibility: hidden;
                    }
                </style>
                """
    st.markdown(hide_streamlit_style, unsafe_allow_html=True)

    st.write(
        """
            # OpenMC geometry plot

            ### ⚛ A geometry plotting user interface for OpenMC.

            🐍 Run this app locally with Python ```pip install openmc_geometry_plot``` then run with ```openmc_geometry_plot```

            ⚙ Produce MatPlotLib or Plotly plots in batch with the 🐍 [Python API](https://github.com/fusion-energy/openmc_geometry_plot/tree/master/examples)

            💾 Raise a feature request, report and issue or make a contribution on [GitHub](https://github.com/fusion-energy/openmc_geometry_plot)

            📧 Email feedback to mail@jshimwell.com

            🔗 This package forms part of a more [comprehensive openmc plotting](https://github.com/fusion-energy/openmc_plot) package where geometry, tallies, slices, etc can be plotted and is hosted on [xsplot.com](https://www.xsplot.com/) .
        """
    )
    st.write("<br>", unsafe_allow_html=True)


def main():
    header()

    file_label_col1, file_label_col2 = st.columns([1, 1])
    file_label_col1.write(
        """
            👉 Create your ```openmc.Geometry()``` and export the geometry xml file using ```export_to_xml()```.
        """
    )
    file_label_col2.write(
        """
            👉 Create your DAGMC h5m file using tools like [CAD-to-h5m](https://github.com/fusion-energy/cad_to_dagmc), [STL-to_h5m](https://github.com/fusion-energy/stl_to_h5m) [vertices-to-h5m](https://github.com/fusion-energy/vertices_to_h5m), [Brep-to-h5m](https://github.com/fusion-energy/brep_to_h5m) or the [Cubit](https://coreform.com/products/coreform-cubit/) [Plugin](https://github.com/svalinn/Cubit-plugin)
        """
    )
    file_col1, file_col2 = st.columns([1, 1])
    geometry_xml_file = file_col1.file_uploader(
        "Upload your geometry.xml", type=["xml"]
    )
    dagmc_file = file_col2.file_uploader("Upload your DAGMC h5m", type=["h5m"])

    my_geometry = None

    if dagmc_file is None and geometry_xml_file is None:
        new_title = '<center><p style="font-family:sans-serif; color:Red; font-size: 30px;">Upload your geometry.xml or DAGMC h5m file</p></center>'
        st.markdown(new_title, unsafe_allow_html=True)

        sub_title = '<center><p> Not got geometry files handy? Download an example <a href="https://raw.githubusercontent.com/fusion-energy/openmc_plot/main/examples/tokamak/geometry.xml" download>geometry.xml</a> or DAGMC h5m file</p></center>'
        st.markdown(sub_title, unsafe_allow_html=True)

    # DAGMC route
    elif dagmc_file is not None and geometry_xml_file is not None:

        save_uploadedfile(dagmc_file)
        save_uploadedfile(geometry_xml_file)

        bound_dag_univ = openmc.DAGMCUniverse(
            filename=dagmc_file.name
        ).bounded_universe()
        my_geometry = openmc.Geometry(root=bound_dag_univ)

        dag_universe = my_geometry.get_dagmc_universe()

        mat_ids = range(0, len(dag_universe.material_names) + 1)
        # mat_names = dag_universe.material_names

        if len(mat_ids) >= 1:
            set_mat_ids = set(mat_ids)
        else:
            set_mat_ids = ()

        # set_cell_ids = set(di.get_volumes_from_h5m(dagmc_file.name))
        set_cell_ids = list(range(1, dag_universe.n_cells + 1))
        set_mat_names = set(dag_universe.material_names)
        all_cell_names = set_cell_ids
        set_cell_names = set(all_cell_names)

    elif dagmc_file is not None and geometry_xml_file is None:

        save_uploadedfile(dagmc_file)

        # make a basic openmc geometry
        bound_dag_univ = openmc.DAGMCUniverse(
            filename=dagmc_file.name
        ).bounded_universe()
        my_geometry = openmc.Geometry(root=bound_dag_univ)

        dag_universe = my_geometry.get_dagmc_universe()

        # find all material names
        mat_ids = range(0, len(dag_universe.material_names) + 1)

        if len(mat_ids) >= 1:
            set_mat_ids = set(mat_ids)
        else:
            set_mat_ids = ()

        # set_cell_ids = set(di.get_volumes_from_h5m(dagmc_file.name))
        set_cell_ids = list(range(1, dag_universe.n_cells + 1))
        set_mat_names = set(dag_universe.material_names)
        all_cell_names = set_cell_ids
        set_cell_names = set(all_cell_names)

    # CSG route
    elif dagmc_file is None and geometry_xml_file is not None:
        save_uploadedfile(geometry_xml_file)

        tree = ET.parse(geometry_xml_file.name)

        root = tree.getroot()
        all_cells = root.findall("cell")
        mat_ids = []

        for cell in all_cells:
            if "material" in cell.keys():
                if cell.get("material") == "void":
                    mat_ids.append(0)
                    print(f"material for cell {cell} is void")
                else:
                    mat_ids.append(int(cell.get("material")))

        if len(mat_ids) >= 1:
            set_mat_ids = set(mat_ids)
        else:
            set_mat_ids = ()

        set_mat_names = set_mat_ids  # can't find material names in CSG with just the geometry xml as we don't have material names

        my_mats = openmc.Materials()
        for mat_id in set_mat_ids:
            new_mat = openmc.Material()
            new_mat.id = mat_id
            new_mat.add_nuclide("He4", 1)
            # adds a single nuclide that is in minimal cross section xml to avoid material failing
            my_mats.append(new_mat)

        my_geometry = openmc.Geometry.from_xml(
            path=geometry_xml_file.name, materials=my_mats
        )
        all_cell_ids = []
        all_cell_names = []
        all_cells = my_geometry.get_all_cells()
        for cell_id, cell in all_cells.items():
            all_cell_ids.append(cell.id)
            all_cell_names.append(cell.name)
        set_cell_ids = set(all_cell_ids)
        set_cell_names = set(all_cell_names)

    if my_geometry:
        print("geometry is set to something so attempting to plot")
        bb = my_geometry.bounding_box

        col1, col2 = st.columns([1, 3])

        view_direction = col1.selectbox(
            label="View direction",
            options=("z", "x", "y"),
            index=0,
            key="geometry_view_direction",
            help="Setting the direction of view automatically sets the horizontal and vertical axis used for the plot.",
        )
        backend = col1.selectbox(
            label="Ploting backend",
            options=("matplotlib", "plotly"),
            index=0,
            key="geometry_ploting_backend",
            help="Create png images with MatPlotLib or HTML plots with Plotly",
        )
        outline = col1.selectbox(
            label="Outline",
            options=("materials", "cells", None),
            index=0,
            key="outline",
            help="Allows an outline to be drawn around the cells or materials, select None for no outline",
        )
        color_by = col1.selectbox(
            label="Color by",
            options=("materials", "cells"),
            index=0,
            key="color_by",
            help="Should the plot be colored by material or by cell",
        )
        plot_left, plot_right = None, None
        plot_bottom, plot_top = None, None
        x_min, x_max = None, None
        y_min, y_max = None, None

        x_index = {"z": 0, "y": 0, "x": 1}[view_direction]
        y_index = {"z": 1, "y": 2, "x": 2}[view_direction]
        slice_index = {"z": 2, "y": 1, "x": 0}[view_direction]

        if np.isinf(bb[0][x_index]) or np.isinf(bb[1][x_index]):
            x_min = col1.number_input(label="minimum vertical axis value", key="x_min")
            x_max = col1.number_input(label="maximum vertical axis value", key="x_max")
        else:
            x_min = float(bb[0][x_index])
            x_max = float(bb[1][x_index])

        # y axis is y values
        if np.isinf(bb[0][y_index]) or np.isinf(bb[1][y_index]):
            y_min = col1.number_input(label="minimum vertical axis value", key="y_min")
            y_max = col1.number_input(label="maximum vertical axis value", key="y_max")
        else:
            y_min = float(bb[0][y_index])
            y_max = float(bb[1][y_index])

        # slice axis is z
        if np.isinf(bb[0][slice_index]) or np.isinf(bb[1][slice_index]):
            slice_min = col1.number_input(label="minimum slice value", key="slice_min")
            slice_max = col1.number_input(label="maximum slice value", key="slice_max")
        else:
            slice_min = float(bb[0][slice_index])
            slice_max = float(bb[1][slice_index])

        if isinstance(x_min, float) and isinstance(x_max, float):
            plot_right, plot_left = col1.slider(
                label="Left and right values for the horizontal axis",
                min_value=x_min,
                max_value=x_max,
                value=(x_min, x_max),
                key="left_right_slider",
                help="Set the lowest visible value and highest visible value on the horizontal axis",
            )

        if isinstance(y_min, float) and isinstance(y_max, float):
            plot_bottom, plot_top = col1.slider(
                label="Bottom and top values for the vertical axis",
                min_value=y_min,
                max_value=y_max,
                value=(y_min, y_max),
                key="bottom_top_slider",
                help="Set the lowest visible value and highest visible value on the vertical axis",
            )
        if isinstance(slice_min, float) and isinstance(slice_max, float):
            slice_value = col1.slider(
                label="Slice value",
                min_value=slice_min,
                max_value=slice_max,
                value=(slice_min + slice_max) / 2,
                key="slice_slider",
                help="Set the value of the slice axis",
            )

        pixels_across = col1.number_input(
            label="Number of horizontal pixels",
            value=500,
            help="Increasing this value increases the image resolution but also requires longer to create the image",
        )

        selected_color_map = col1.selectbox(
            label="Color map", options=colormaps(), index=82
        )  # index 81 is tab20c

        if color_by == "materials":

            cmap = cm.get_cmap(selected_color_map, len(set_mat_ids))
            initial_hex_color = []
            for i in range(cmap.N):
                rgba = cmap(i)
                # rgb2hex accepts rgb or rgba
                initial_hex_color.append(colors.rgb2hex(rgba))

            for c, id in enumerate(set_mat_ids):
                # todo add
                st.color_picker(
                    f"Color of material with id {id}",
                    key=f"mat_{id}",
                    value=initial_hex_color[c],
                )

            my_colors = {}
            for id in set_mat_ids:
                hex_color = st.session_state[f"mat_{id}"].lstrip("#")
                RGB = tuple(int(hex_color[i : i + 2], 16) / 255 for i in (0, 2, 4))
                my_colors[id] = RGB

        elif color_by == "cells":
            cmap = cm.get_cmap(selected_color_map, len(set_cell_ids))
            initial_hex_color = []
            for i in range(cmap.N):
                rgba = cmap(i)
                # rgb2hex accepts rgb or rgba
                initial_hex_color.append(colors.rgb2hex(rgba))

            for c, (cell_id, cell_name) in enumerate(zip(set_cell_ids, all_cell_names)):
                if cell_name in ["", None]:
                    cell_name = "not set"
                st.color_picker(
                    f"Color of cell id {cell_id}, cell name {cell_name}",
                    key=f"cell_{cell_id}",
                    value=initial_hex_color[c],
                )

            my_colors = {0: (1, 1, 1)}  # adding entry for void cells
            for id in set_cell_ids:
                hex_color = st.session_state[f"cell_{id}"].lstrip("#")
                RGB = tuple(int(hex_color[i : i + 2], 16) / 255 for i in (0, 2, 4))
                my_colors[id] = RGB

        title = col1.text_input(
            "Plot title",
            help="Optionally set your own title for the plot",
            value=f"Slice through OpenMC geometry with view direction {view_direction}",
        )

        if (
            isinstance(plot_left, float)
            and isinstance(plot_right, float)
            and isinstance(plot_top, float)
            and isinstance(plot_bottom, float)
        ):
            if color_by == "cells":
                color_data_slice = my_geometry.get_slice_of_cell_ids(
                    view_direction=view_direction,
                    plot_left=plot_left,
                    plot_right=plot_right,
                    plot_top=plot_top,
                    plot_bottom=plot_bottom,
                    pixels_across=pixels_across,
                    slice_value=slice_value,
                )
            elif color_by == "materials":
                color_data_slice = my_geometry.get_slice_of_material_ids(
                    view_direction=view_direction,
                    plot_left=plot_left,
                    plot_right=plot_right,
                    plot_top=plot_top,
                    plot_bottom=plot_bottom,
                    pixels_across=pixels_across,
                    slice_value=slice_value,
                )

            (xlabel, ylabel) = my_geometry.get_axis_labels(
                view_direction=view_direction
            )
            if outline is not None:
                # gets unique levels for outlines contour plot
                # this can be avoided if outline is the same as the color data
                if outline == color_by:
                    outline_data_slice = color_data_slice
                elif outline == "cells":
                    outline_data_slice = my_geometry.get_slice_of_cell_ids(
                        view_direction=view_direction,
                        plot_left=plot_left,
                        plot_right=plot_right,
                        plot_top=plot_top,
                        plot_bottom=plot_bottom,
                        pixels_across=pixels_across,
                        slice_value=slice_value,
                    )
                elif outline == "materials":
                    outline_data_slice = my_geometry.get_slice_of_material_ids(
                        view_direction=view_direction,
                        plot_left=plot_left,
                        plot_right=plot_right,
                        plot_top=plot_top,
                        plot_bottom=plot_bottom,
                        pixels_across=pixels_across,
                        slice_value=slice_value,
                    )
                else:
                    raise ValueError(
                        f"outline can only be cells or materials, not {outline}"
                    )

            if backend == "matplotlib":

                extent = my_geometry.get_plot_extent(
                    plot_left,
                    plot_right,
                    plot_bottom,
                    plot_top,
                    slice_value,
                    bb,
                    view_direction,
                )[
                    :-1
                ]  # slice value is returned in the function so removing with -1

                bounds = list(my_colors.keys())
                color_values = list(my_colors.values())

                mat_cmap = colors.ListedColormap(color_values)
                # our material ids are set to be 1 and 2. void space is 0
                # this +1 is needed as the bounds must be larger that the value to include the values
                bounds.append(bounds[-1] + 1)

                mat_norm = colors.BoundaryNorm(bounds, mat_cmap.N)

                plt.imshow(
                    color_data_slice,
                    extent=extent,
                    interpolation="none",
                    norm=mat_norm,  # needed for colors
                    cmap=mat_cmap,  # needed for colors
                )

                plt.xlabel(xlabel)
                plt.ylabel(ylabel)
                plt.title(title)

                if outline is not None:
                    levels = np.unique(
                        [item for sublist in outline_data_slice for item in sublist]
                    )
                    plt.contour(
                        outline_data_slice,
                        origin="upper",
                        colors="k",
                        linestyles="solid",
                        levels=levels,
                        linewidths=0.5,
                        extent=extent,
                    )

                plt.savefig("openmc_plot_geometry_image.png")
                col2.pyplot(plt)
                # col2.image("openmc_plot_geometry_image.png", use_column_width="always")

                with open("openmc_plot_geometry_image.png", "rb") as file:
                    col1.download_button(
                        label="Download image",
                        data=file,
                        file_name="openmc_plot_geometry_image.png",
                        mime="image/png",
                    )
            else:

                data = [
                    go.Heatmap(
                        z=color_data_slice,
                        showscale=False,
                        colorscale="viridis",
                        x0=plot_left,
                        dx=abs(plot_left - plot_right) / (len(color_data_slice[0]) - 1),
                        y0=plot_bottom,
                        dy=abs(plot_bottom - plot_top) / (len(color_data_slice) - 1),
                        # colorbar=dict(title=dict(side="right", text=cbar_label)),
                        # text = material_ids,
                        # hovertemplate=
                        # # 'material ID = %{z}<br>'+
                        # "Cell ID = %{z}<br>" +
                        # # '<br>%{text}<br>'+
                        # xlabel[:2].title()
                        # + ": %{x} cm<br>"
                        # + ylabel[:2].title()
                        # + ": %{y} cm<br>",
                    )
                ]

                if outline is not None:

                    data.append(
                        go.Contour(
                            z=outline_data_slice,
                            x0=plot_left,
                            dx=abs(plot_left - plot_right)
                            / (len(outline_data_slice[0]) - 1),
                            y0=plot_bottom,
                            dy=abs(plot_bottom - plot_top)
                            / (len(outline_data_slice) - 1),
                            contours_coloring="lines",
                            line_width=1,
                            colorscale=[[0, "rgb(0, 0, 0)"], [1.0, "rgb(0, 0, 0)"]],
                            showscale=False,
                        )
                    )

                plot = go.Figure(data=data)

                plot.update_layout(
                    xaxis={"title": xlabel},
                    # reversed autorange is required to avoid image needing rotation/flipping in plotly
                    yaxis={"title": ylabel, "autorange": "reversed"},
                    title=title,
                    autosize=False,
                    height=800,
                )
                plot.update_yaxes(
                    scaleanchor="x",
                    scaleratio=1,
                )

                plot.write_html("openmc_plot_geometry_image.html")

                with open("openmc_plot_geometry_image.html", "rb") as file:
                    col1.download_button(
                        label="Download image",
                        data=file,
                        file_name="openmc_plot_geometry_image.html",
                        mime=None,
                    )
                col2.plotly_chart(plot, use_container_width=True)

            col2.write("Model info")
            col2.write(f"Material IDS found {set_mat_ids}")
            col2.write(f"Material names found {set_mat_names}")
            col2.write(f"Cell IDS found {set_cell_ids}")
            col2.write(f"Cell names found {set_cell_names}")
            col2.write(
                f"Bounding box lower left x={bb[0][0]} y={bb[0][1]} z={bb[0][2]}"
            )
            col2.write(
                f"Bounding box upper right x={bb[1][0]} y={bb[1][1]} z={bb[1][2]}"
            )


if __name__ == "__main__":
    main()



from importlib.metadata import version, PackageNotFoundError

try:
    __version__ = version("openmc_geometry_plot")
except PackageNotFoundError:
    from setuptools_scm import get_version

    __version__ = get_version(root="..", relative_to=__file__)

__all__ = ["__version__"]

from .core import *



import openmc
import openmc_geometry_plot
import numpy as np


def test_get_slice_of_material_ids():
    openmc_material = openmc.Material()
    openmc_material.id = 1

    # my_materials = openmc.Materials([openmc_material])

    spheres = [openmc.Sphere(r=r) for r in [10, 20, 30, 40, 50]]

    spheres[-1].boundary_type = "vacuum"

    regions = openmc.model.subdivide(spheres)

    cells = [openmc.Cell(fill=openmc_material, region=r) for r in regions[:-1]]

    my_geometry = openmc.Geometry(cells)

    slice_data = my_geometry.get_slice_of_material_ids(view_direction="x")

    assert np.array(slice_data).shape == (500, 500)
    for rows in slice_data:
        for value in rows:
            assert value in [1, 0]



import matplotlib.pylab as plt
import numpy as np
import openmc
import openmc_geometry_plot  # adds plot_axis_slice to openmc.Geometry


height = 150
outer_radius = 50
thickness = 10

outer_cylinder = openmc.ZCylinder(r=outer_radius, boundary_type="vacuum")
inner_cylinder = openmc.ZCylinder(r=outer_radius - thickness, boundary_type="vacuum")
inner_top = openmc.ZPlane(z0=height * 0.5, boundary_type="vacuum")
inner_bottom = openmc.ZPlane(z0=-height * 0.5, boundary_type="vacuum")
outer_top = openmc.ZPlane(z0=(height * 0.5) + thickness, boundary_type="vacuum")
outer_bottom = openmc.ZPlane(z0=(-height * 0.5) - thickness, boundary_type="vacuum")

steel = openmc.Material()
steel.set_density("g/cm3", 7.75)
steel.add_element("Fe", 0.95, percent_type="wo")
steel.add_element("C", 0.05, percent_type="wo")

mats = openmc.Materials([steel])

cylinder_region = -outer_cylinder & +inner_cylinder & -inner_top & +inner_bottom
cylinder_cell = openmc.Cell(region=cylinder_region)
cylinder_cell.fill = steel

top_cap_region = -outer_top & +inner_top & -outer_cylinder
top_cap_cell = openmc.Cell(region=top_cap_region)
top_cap_cell.fill = steel

bottom_cap_region = +outer_bottom & -inner_bottom & -outer_cylinder
bottom_cap_cell = openmc.Cell(region=bottom_cap_region)
bottom_cap_cell.fill = steel

inner_void_region = -inner_cylinder & -inner_top & +inner_bottom
inner_void_cell = openmc.Cell(region=inner_void_region)

my_geometry = openmc.Geometry(
    [inner_void_cell, cylinder_cell, top_cap_cell, bottom_cap_cell]
)


# example code for plotting materials of the geometry with an outline


xlabel, ylabel = my_geometry.get_axis_labels(view_direction="x")
plt.xlabel(xlabel)
plt.ylabel(ylabel)

plot_extent = my_geometry.get_mpl_plot_extent(view_direction="x")


data_slice = my_geometry.get_slice_of_cell_ids(view_direction="x")
# plots the cells with randomly assigned colors
plt.imshow(
    data_slice,
    extent=plot_extent,
    interpolation="none",
)

# gets unique levels for outlines contour plot
levels = np.unique([item for sublist in data_slice for item in sublist])

# plots the outline of the cells
plt.contour(
    data_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=levels,
    linewidths=0.5,
    extent=plot_extent,
)

plt.show()



import openmc
import openmc_geometry_plot
import numpy as np


def test_get_slice_of_material_ids():
    openmc_material = openmc.Material()
    openmc_material.id = 1

    # my_materials = openmc.Materials([openmc_material])

    surf_1 = openmc.Sphere(r=10)
    surf_2 = openmc.Sphere(r=20)
    surf_3 = openmc.Sphere(r=30, boundary_type="vacuum")

    region_1 = -surf_1
    region_2 = +surf_1 & -surf_2
    region_3 = +surf_2 & -surf_3

    cell1 = openmc.Cell(fill=openmc_material, region=region_1)
    cell1.id = 10
    cell2 = openmc.Cell(region=region_2)
    cell2.id = 20
    cell3 = openmc.Cell(fill=openmc_material, region=region_3)
    cell3.id = 40

    my_geometry = openmc.Geometry([cell1, cell2, cell3])

    slice_data = my_geometry.get_slice_of_cell_ids(view_direction="x")

    assert np.array(slice_data).shape == (500, 500)
    for rows in slice_data:
        for value in rows:
            assert value in [0, 10, 20, 40]



import numpy as np
import matplotlib.pylab as plt
import openmc
import openmc_geometry_plot  # adds functions to openmc.Geometry
from matplotlib import colors

# MATERIALS

mat1 = openmc.Material()
mat1.set_density("g/cm3", 1)
mat1.id = 1
mat1.add_element("Pb", 1)

mat2 = openmc.Material()
mat2.id = 2
mat2.set_density("g/cm3", 1)
mat2.add_element("Fe", 1)

mats = openmc.Materials([mat1, mat2])

# GEOMETRY

# surfaces
surf1 = openmc.Sphere(r=100)
surf2 = openmc.Sphere(r=200)
surf3 = openmc.Sphere(r=300, boundary_type="vacuum")

# regions
region1 = -surf1
region2 = -surf2 & +surf1
region3 = +surf2 & -surf3

# cells
cell1 = openmc.Cell(region=region1)
cell2 = openmc.Cell(region=region2)
cell2.fill = mat2
cell3 = openmc.Cell(region=region3)
cell3.fill = mat1

my_geometry = openmc.Geometry([cell1, cell2, cell3])


# this part plots the geometry with colors

data_slice = my_geometry.get_slice_of_material_ids(view_direction="z")

xlabel, ylabel = my_geometry.get_axis_labels(view_direction="z")
plt.xlabel(xlabel)
plt.ylabel(ylabel)

# gets unique levels for outlines contour plot and for the color scale
levels = np.unique([item for sublist in data_slice for item in sublist])

cmap = colors.ListedColormap(["white", "red", "blue"])
# our material ids are set to be 1 and 2. void space is 0
bounds = [0, 1, 2, 3]
norm = colors.BoundaryNorm(bounds, cmap.N)

plot_extent = my_geometry.get_mpl_plot_extent(view_direction="z")

# shows slice of material ids colored with colour map
plt.imshow(
    data_slice,
    extent=plot_extent,
    interpolation="none",
    norm=norm,  # needed for colors
    cmap=cmap,  # needed for colors
)

# shows outlines of materials
plt.contour(
    data_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=levels,
    linewidths=0.5,
    extent=plot_extent,
)

plt.show()



import numpy as np
import matplotlib.pylab as plt
import openmc
import openmc_geometry_plot  # adds functions to openmc.Geometry
from matplotlib import colors
import matplotlib.patches as mpatches

# MATERIALS

mat1 = openmc.Material()
mat1.set_density("g/cm3", 1)
mat1.id = 1
mat1.add_element("Pb", 1)

mat2 = openmc.Material()
mat2.id = 2
mat2.set_density("g/cm3", 1)
mat2.add_element("Fe", 1)

mats = openmc.Materials([mat1, mat2])

# GEOMETRY

# surfaces
vessel_inner = openmc.Sphere(r=100)
first_wall_outer_surface = openmc.Sphere(r=200)
breeder_blanket_outer_surface = openmc.Sphere(r=300, boundary_type="vacuum")

# cells
inner_vessel_region = -vessel_inner
inner_vessel_cell = openmc.Cell(region=inner_vessel_region)

first_wall_region = -first_wall_outer_surface & +vessel_inner
first_wall_cell = openmc.Cell(region=first_wall_region)
first_wall_cell.fill = mat2

breeder_blanket_region = +first_wall_outer_surface & -breeder_blanket_outer_surface
breeder_blanket_cell = openmc.Cell(region=breeder_blanket_region)
breeder_blanket_cell.fill = mat1

my_geometry = openmc.Geometry(
    [inner_vessel_cell, first_wall_cell, breeder_blanket_cell]
)


# these three lines are functionality added by the openmc_geometry_plot
xlabel, ylabel = my_geometry.get_axis_labels(view_direction="x")
data_slice = my_geometry.get_slice_of_material_ids(view_direction="x")

# gets unique levels for outlines contour plot and for the color scale
levels = np.unique([item for sublist in data_slice for item in sublist])

cmap = colors.ListedColormap(["white", "red", "blue"])
# our material ids are set to be 1 and 2. void space is 0
bounds = [0, 1, 2, 3]
norm = colors.BoundaryNorm(bounds, cmap.N)

plot_extent = my_geometry.get_mpl_plot_extent(view_direction="x")

plt.xlabel(xlabel)
plt.ylabel(ylabel)

im = plt.imshow(
    data_slice,
    extent=plot_extent,
    interpolation="none",
    norm=norm,  # needed for colors
    cmap=cmap,  # needed for colors
)

# gets the colors of the values according to the color map used by imshow
colors = [im.cmap(im.norm(value)) for value in levels]

# these will be the labels used for each material id
labels = {0: "vacuum", 1: "mat1", 2: "mat2"}

patches = [
    mpatches.Patch(color=colors[i], label=labels[levels[i]]) for i in range(len(levels))
]

plt.legend(handles=patches, bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.0)

plt.contour(
    data_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=levels,
    linewidths=0.5,
    extent=plot_extent,
)

plt.show()



import matplotlib.pylab as plt
import numpy as np
import openmc
import openmc_geometry_plot  # adds plot_axis_slice to openmc.Geometry


breeder_material = openmc.Material()
breeder_material.add_element("Li", 1.0)
breeder_material.set_density("g/cm3", 0.01)

copper_material = openmc.Material()
copper_material.set_density("g/cm3", 0.01)
copper_material.add_element("Cu", 1.0)

eurofer_material = openmc.Material()
eurofer_material.set_density("g/cm3", 0.01)
eurofer_material.add_element("Fe", 1)

# surfaces
central_sol_surface = openmc.ZCylinder(r=100)
central_shield_outer_surface = openmc.ZCylinder(r=110)
port_hole = openmc.Sphere(r=60, x0=500)
upper_port_hole = openmc.Sphere(r=100, z0=500)
vessel_inner_surface = openmc.Sphere(r=500)
first_wall_outer_surface = openmc.Sphere(r=510)
breeder_blanket_outer_surface = openmc.Sphere(r=610, boundary_type="vacuum")

# cells
central_sol_region = (
    -central_sol_surface & -breeder_blanket_outer_surface & +upper_port_hole
)
central_sol_cell = openmc.Cell(region=central_sol_region)
central_sol_cell.fill = copper_material

central_shield_region = (
    +central_sol_surface
    & -central_shield_outer_surface
    & -breeder_blanket_outer_surface
    & +upper_port_hole
)
central_shield_cell = openmc.Cell(region=central_shield_region)
central_shield_cell.fill = eurofer_material

inner_vessel_region = (
    -vessel_inner_surface
    & +central_shield_outer_surface
    & +port_hole
    & +upper_port_hole
)
inner_vessel_cell = openmc.Cell(region=inner_vessel_region)
# no material set as default is vacuum

upper_port_hole_region = -upper_port_hole
upper_port_hole_cell = openmc.Cell(region=upper_port_hole_region)

port_hole_region = -port_hole
port_hole_cell = openmc.Cell(region=port_hole_region)
# no material set as default is vacuum

first_wall_region = (
    -first_wall_outer_surface & +vessel_inner_surface & +port_hole & +upper_port_hole
)
first_wall_cell = openmc.Cell(region=first_wall_region)
first_wall_cell.fill = eurofer_material

breeder_blanket_region = (
    +first_wall_outer_surface
    & -breeder_blanket_outer_surface
    & +central_shield_outer_surface
    & +port_hole
    & +upper_port_hole
)
breeder_blanket_cell = openmc.Cell(region=breeder_blanket_region)
breeder_blanket_cell.fill = breeder_material

my_geometry = openmc.Geometry(
    [
        central_sol_cell,
        central_shield_cell,
        inner_vessel_cell,
        first_wall_cell,
        breeder_blanket_cell,
        port_hole_cell,
        upper_port_hole_cell,
    ]
)


# example code for plotting materials of the geometry with an outline

xlabel, ylabel = my_geometry.get_axis_labels(view_direction="x")
plt.xlabel(xlabel)
plt.ylabel(ylabel)

plot_extent = my_geometry.get_mpl_plot_extent(view_direction="x")

data_slice = my_geometry.get_slice_of_material_ids(view_direction="x")
# shows the materials with randomly assigned colors
plt.imshow(
    data_slice,
    extent=plot_extent,
    interpolation="none",
)

# gets unique levels for outlines contour plot
levels = np.unique([item for sublist in data_slice for item in sublist])

# shows the outline of the materials
plt.contour(
    data_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=levels,
    linewidths=0.5,
    extent=plot_extent,
)

plt.show()



import matplotlib.pylab as plt
import numpy as np
import openmc
import openmc_geometry_plot  # adds data slicing functions to openmc.Geometry


material_1 = openmc.Material()
material_1.set_density("g/cm3", 0.01)
material_1.add_element("Ag", 1)

material_2 = openmc.Material()
material_2.set_density("g/cm3", 0.01)
material_2.add_element("Li", 1)

# surfaces
surface_1 = openmc.ZCylinder(r=50)
surface_2 = openmc.ZCylinder(r=100)

region_1 = -surface_1
region_2 = -surface_2 & +surface_1

cell_1 = openmc.Cell(region=region_1)
cell_1.fill = material_1

cell_2 = openmc.Cell(region=region_2)
cell_2.fill = material_2

universe = openmc.Universe(cells=[cell_1, cell_2])
my_geometry = openmc.Geometry(universe)


# example code for plotting materials of the geometry with an outline

# defines the plot extent
plot_left = -100
plot_right = 100
plot_bottom = 200
plot_top = -200

data_slice = my_geometry.get_slice_of_material_ids(
    view_direction="x",
    plot_left=plot_left,
    plot_right=plot_right,
    plot_bottom=plot_bottom,
    plot_top=plot_top,
)

xlabel, ylabel = my_geometry.get_axis_labels(view_direction="x")
plt.xlabel(xlabel)
plt.ylabel(ylabel)

# plots of cells with randomly assigned colors
plt.imshow(
    data_slice,
    extent=(plot_left, plot_right, plot_bottom, plot_top),
    interpolation="none",
)

# gets unique levels for outlines contour plot
levels = np.unique([item for sublist in data_slice for item in sublist])

# plots the outline of the cells
plt.contour(
    data_slice,
    origin="upper",
    colors="k",
    linestyles="solid",
    levels=levels,
    linewidths=0.5,
    extent=(plot_left, plot_right, plot_bottom, plot_top),
)

plt.show()
